//
//  InterfaceController.swift
//  Soccer WatchKit Extension
//
//  Created by Jeff Kelley on 8/2/15.
//  Copyright © 2015 The Pragmatic Bookshelf. All rights reserved.
//

import WatchKit
import Foundation
import CoreMotion

class InterfaceController: WKInterfaceController {
    
    // MARK:- UI Elements
    
    @IBOutlet weak var soccerBall: WKInterfaceGroup!
    @IBOutlet weak var soccerField: WKInterfaceGroup!
    
    // MARK:-
    
    lazy var dataQueue = dispatch_queue_create("accelerometerData", 
        DISPATCH_QUEUE_CONCURRENT)
    
    private var _accelerometerData = CMAcceleration(x: 0, y: 0, z: 0) 
    
    var accelerometerData: CMAcceleration
        {
        get {
            var data: CMAcceleration = CMAcceleration(x: 0, y: 0, z: 0)
            
            dispatch_sync(dataQueue) { 
                data = self._accelerometerData
            }
            
            return data
        }
        set {
            dispatch_barrier_async(dataQueue) { 
                self._accelerometerData = newValue
            }
        }
    }

    let maximumSpeed: Double = 5
    
    var ballSpeed: (x: Double, y: Double) = (0, 0) {
        didSet {
            if ballSpeed.x > maximumSpeed {
                ballSpeed.x = maximumSpeed
            }
            
            if ballSpeed.x < -maximumSpeed {
                ballSpeed.x = -maximumSpeed
            }
            
            if ballSpeed.y > maximumSpeed {
                ballSpeed.y = maximumSpeed
            }
            
            if ballSpeed.y < -maximumSpeed {
                ballSpeed.y = -maximumSpeed
            }
        }
    }
    
    lazy var motionManager = CMMotionManager()
    lazy var motionQueue = NSOperationQueue()

    // MARK:- WKInterfaceController Lifecycle
    
    var updateTimer: NSTimer?
    
    override func didAppear() {
        super.didAppear()
        
        guard motionManager.accelerometerAvailable else { return }
        
        motionManager.startAccelerometerUpdatesToQueue(motionQueue) { data, _ in
            guard let data = data else { return }
            
            self.accelerometerData = data.acceleration
        }
        
        if updateTimer == nil {
            updateTimer = NSTimer.scheduledTimerWithTimeInterval(1.0 / 30.0,
                target: self,
                selector: "updateAccelerometerData:",
                userInfo: nil,
                repeats: true)
        }
    }
    
    override func didDeactivate() {
        super.didDeactivate()
        
        updateTimer?.invalidate()
        updateTimer = nil
        
        if motionManager.accelerometerActive {
            motionManager.stopAccelerometerUpdates()
        }
    }
    
    // MARK:- Accelerometer Updates
    
    func updateAccelerometerData(timer: NSTimer) {
        let data = accelerometerData
        
        ballSpeed.x += data.x
        ballSpeed.y += -data.y
        
        currentInsets.left += CGFloat(ballSpeed.x)
        currentInsets.top += CGFloat(ballSpeed.y)

        soccerField.setContentInset(currentInsets)
    }
    
    // MARK:- Properties
    
    let ballSize: CGSize = CGSize(width: 20, height: 20)
    
    var currentInsets: UIEdgeInsets = UIEdgeInsetsZero {
        didSet {
            if currentInsets.left + ballSize.width > soccerFieldSize.width {
                currentInsets.left = soccerFieldSize.width - ballSize.width
            }
            
            if currentInsets.left < 0 {
                currentInsets.left = 0
            }
            
            if currentInsets.top + ballSize.height > soccerFieldSize.height {
                currentInsets.top = soccerFieldSize.height - ballSize.height
            }
            
            if currentInsets.top < 0 {
                currentInsets.top = 0
            }
        }
    }
    
    var soccerFieldSize: CGSize {
        return contentFrame.size
    }

    // MARK:-

}
