//
//  RunTimerInterfaceController.swift
//  TapALap
//
//  Created by Jeff Kelley on 10/23/15.
//  Copyright © 2015 The Pragmatic Bookshelf. All rights reserved.
//

import WatchKit
import Foundation
import HealthKit


class RunTimerInterfaceController: WKInterfaceController, HKWorkoutSessionDelegate {

    @IBOutlet weak var distanceLabel: WKInterfaceLabel!
    @IBOutlet weak var caloriesBurnedLabel: WKInterfaceLabel!
    
    lazy var track: Track = Track(name: "Track", lapDistance:  500)
    
    var lapTimes: [NSTimeInterval]?
    var startDate: NSDate?
    
    override func awakeWithContext(context: AnyObject?) {
        super.awakeWithContext(context)
        
        // Configure interface objects here.
    }

    lazy var healthStore = HKHealthStore()

    override func willActivate() {
        super.willActivate()
        
        askForHealthKitPermission()
        
        if lapTimes == nil || startDate == nil {
            let userDefaults = NSUserDefaults.standardUserDefaults()
            
            if let date = userDefaults.objectForKey("StartDate") as? NSDate,
                times = userDefaults.objectForKey("LapTimes") as? [NSTimeInterval] {
                    startDate = date
                    lapTimes = times
            }
            else {
                startRun()
            }
        }
        
        updateDistanceLabel()
        
        // Don’t delete the rest of this method—it’s just removed to save space!
        
    }
    
    func askForHealthKitPermission() {
        guard HKHealthStore.isHealthDataAvailable() else { return }
        
        guard let caloriesBurnedType = HKObjectType.quantityTypeForIdentifier(
            HKQuantityTypeIdentifierActiveEnergyBurned),
            distanceRunType = HKObjectType.quantityTypeForIdentifier(
                HKQuantityTypeIdentifierDistanceWalkingRunning) else { return }
        
        let shareTypes = Set([HKObjectType.workoutType(), caloriesBurnedType,
            distanceRunType])
        
        let readTypes = Set([caloriesBurnedType])
        
        healthStore.requestAuthorizationToShareTypes(shareTypes,
            readTypes: readTypes,
            completion: { (success: Bool, error: NSError?) in
                if let error = error where !success {
                    print("Error authorizing HealthKit: " +
                        error.localizedDescription + ".")
                }
                else if !success {
                    print("You didn’t authorize HealthKit. Open the Health app on "
                        + "your iPhone to give TapALap the correct permissions.")
                }
        })
    }
    
    func startRun() {
        lapTimes = []
        startDate = NSDate()
        
        let userDefaults = NSUserDefaults.standardUserDefaults()
        userDefaults.setObject(startDate, forKey: "StartDate")
        userDefaults.setObject(lapTimes, forKey: "LapTimes")
        userDefaults.synchronize()
        
        startWorkoutSession()
    }

    override func didDeactivate() {
        // This method is called when watch view controller is no longer visible
        super.didDeactivate()
    }

    @IBAction func stopRunButtonPressed() {
        endRun()
    }
    
    func endRun() {
        if let query = currentCaloriesBurnedQuery {
            healthStore.stopQuery(query)
            currentCaloriesBurnedQuery = nil
        }
        
        if let session = currentWorkoutSession {
            healthStore.endWorkoutSession(session)
            saveWorkoutSession(session)
            currentWorkoutSession = nil
        }
        
        let names = ["GoRunning", "RunLog"]
        
        let contexts: [AnyObject]?
        
        if let lapTimes = lapTimes, startDate = startDate {
            let distance = track.lapDistance * Double(lapTimes.count)
            
            let run = Run(distance: distance,
                laps: lapTimes,
                startDate: startDate)
            
            let userDefaults = NSUserDefaults.standardUserDefaults()
            userDefaults.removeObjectForKey("LapTimes")
            userDefaults.removeObjectForKey("StartDate")
            userDefaults.synchronize()
            
            contexts = [NSNull(), run]
        }
        else {
            contexts = nil
        }
        
        WKInterfaceController.reloadRootControllersWithNames(names,
            contexts: contexts)
        
        // Don’t delete the rest of this method, it’s merely removed to save space!
        
    }
    
    var currentCaloriesBurned = HKQuantity(unit: HKUnit.kilocalorieUnit(),
        doubleValue: 0.0)
    
    var calorieSamples: [HKQuantitySample] = []
    
    func saveWorkoutSession(session: HKWorkoutSession) {
        guard let caloriesBurnedType = HKObjectType.quantityTypeForIdentifier( //(1)
            HKQuantityTypeIdentifierActiveEnergyBurned),
            distanceRunType = HKObjectType.quantityTypeForIdentifier(
                HKQuantityTypeIdentifierDistanceWalkingRunning) else { return }
        
        // Ensure we have permission to save all types
        for type in [caloriesBurnedType, distanceRunType, .workoutType()] { //(2)
            if healthStore.authorizationStatusForType(type) != .SharingAuthorized {
                return
            }
        }
        
        guard let startDate = startDate, lapTimes = lapTimes //(3)
            where !lapTimes.isEmpty
            else { return }
        
        let endDate = startDate.dateByAddingTimeInterval( //(4)
            lapTimes.reduce(0, combine: +))
        
        let lapDistance = track.lapDistance
        
        let distanceRun = HKQuantity(unit: HKUnit.meterUnit(), //(5)
            doubleValue: lapDistance * Double(lapTimes.count))
        
        let workout = HKWorkout(activityType: HKWorkoutActivityType.Running, //(6)
            startDate: startDate,
            endDate: endDate,
            duration: endDate.timeIntervalSinceDate(startDate),
            totalEnergyBurned: currentCaloriesBurned,
            totalDistance: distanceRun,
            metadata: nil)
        
        let finalCalorieSamples = calorieSamples //(7)
        
        healthStore.saveObject(workout) { [unowned self] success, error in //(8)
            if let error = error where !success {
                print("Failed to save the workout: "
                    + error.localizedDescription)
                return
            }
            
            if success && finalCalorieSamples.count > 0 {
                // Associate the accumulated samples with the workout.
                self.healthStore.addSamples(finalCalorieSamples, //(9)
                    toWorkout: workout) { success, error in
                        if let error = error where !success {
                            print("Failed to add calorie samples to the workout:"
                                + error.localizedDescription)
                        }
                }
            }
            
            if success {
                let lapDistanceQuantity = HKQuantity(unit: HKUnit.meterUnit(),
                    doubleValue: lapDistance)

                var lapStartDate = startDate
                
                var samples: [HKSample] = []
                
                for i in lapTimes.startIndex ..< lapTimes.endIndex { //(10)
                    let lapTime = lapTimes[i]
                    let lapEndDate = lapStartDate.dateByAddingTimeInterval(lapTime)
                    
                    let sample = HKQuantitySample(type: distanceRunType, //(11)
                        quantity: lapDistanceQuantity,
                        startDate: lapStartDate,
                        endDate: lapEndDate,
                        device: HKDevice.localDevice(),
                        metadata: nil)
                    
                    samples.append(sample)
                    
                    lapStartDate = lapEndDate
                }
                
                self.healthStore.addSamples(samples, toWorkout: workout) //(12)
                    { success, error in
                        if let error = error where !success {
                            print("Failed to add distance samples to the workout:"
                                + error.localizedDescription)
                        }
                }
            }
        }
    }
    
    var currentCaloriesBurnedQuery: HKQuery? 
    
    func beginCalorieQuery() {
        guard let calorieType = HKObjectType 
            .quantityTypeForIdentifier(HKQuantityTypeIdentifierActiveEnergyBurned)
            else { return }
        
        let calorieUnit = HKUnit.kilocalorieUnit()
        
        guard let startDate = startDate else { return } 
        
        let datePredicate = HKQuery.predicateForSamplesWithStartDate(startDate, 
            endDate: nil, options: .None)
        
        let processCalorieSamplesFromQuery = { (query: HKAnchoredObjectQuery, 
            samples: [HKSample]?,
            deletedObjects: [HKDeletedObject]?,
            anchor: HKQueryAnchor?,
            error: NSError?) -> Void in
            
            guard let samples = samples as? [HKQuantitySample] else { return }
            
            let newSamples = samples.map { originalSample in 
                HKQuantitySample(type: calorieType,
                    quantity: originalSample.quantity,
                    startDate: originalSample.startDate,
                    endDate: originalSample.endDate)
            }
            
            NSOperationQueue.mainQueue().addOperationWithBlock { [weak self] in 
                guard let initialCaloriesBurned = self?.currentCaloriesBurned
                    .doubleValueForUnit(calorieUnit) else { return }
                
                let newCaloriesBurned = samples.reduce(initialCaloriesBurned) { 
                    $0 + $1.quantity.doubleValueForUnit(calorieUnit)
                }
                
                self?.currentCaloriesBurned = HKQuantity(unit: calorieUnit,
                    doubleValue: newCaloriesBurned)
                
                self?.caloriesBurnedLabel
                    .setText("Calories Burned: \(newCaloriesBurned)")
                
                self?.calorieSamples += newSamples 
            }
        }
        
        let query = HKAnchoredObjectQuery(type: calorieType, 
            predicate: datePredicate,
            anchor: nil,
            limit: Int(HKObjectQueryNoLimit),
            resultsHandler: processCalorieSamplesFromQuery)
        
        query.updateHandler = processCalorieSamplesFromQuery
        
        healthStore.executeQuery(query)
        
        currentCaloriesBurnedQuery = query
    }
    
    lazy var lengthFormatter: NSLengthFormatter = {
        let formatter = NSLengthFormatter()
        formatter.numberFormatter.maximumSignificantDigits = 3
        return formatter
    }()
    
    func updateDistanceLabel() {
        guard let lapTimes = lapTimes where !lapTimes.isEmpty else {
            distanceLabel.setText("No laps finished!")
            return
        }
        
        let distance = track.lapDistance * Double(lapTimes.count)
        
        distanceLabel.setText(lengthFormatter.stringFromMeters(distance))
    }
    
    @IBAction func finishLapButtonPressed() {
        let lapFinishTime = NSDate()
        
        guard let startDate = startDate, lapTimes = lapTimes else { return }
        
        let totalRunDuration = lapFinishTime.timeIntervalSinceDate(startDate)
        
        let cumulativeLapDuration = lapTimes.reduce(0, combine: { $0 + $1 })
        
        let lapDuration = totalRunDuration - cumulativeLapDuration
        
        self.lapTimes?.append(lapDuration)
        
        updateDistanceLabel()
        
        let userDefaults = NSUserDefaults.standardUserDefaults()
        userDefaults.setObject(lapTimes, forKey: "LapTimes")
        userDefaults.synchronize()
    }
    
    // MARK:- Workout Session
    
    var currentWorkoutSession: HKWorkoutSession?
    
    func startWorkoutSession() {
        let session = HKWorkoutSession(activityType: .Running,
            locationType: .Indoor)
        
        session.delegate = self
        
        currentWorkoutSession = session

        healthStore.startWorkoutSession(session)
    }
    
    func workoutSession(workoutSession: HKWorkoutSession,
        didChangeToState toState: HKWorkoutSessionState,
        fromState: HKWorkoutSessionState,
        date: NSDate) {
            // If the workout ends before we call endRun(), end the run.
            if let _ = lapTimes, _ = startDate where toState == .Ended {
                endRun()
            }
            else if toState == .Running {
                beginCalorieQuery()
            }
    }
    
    func workoutSession(workoutSession: HKWorkoutSession,
        didFailWithError error: NSError) {
            print("Workout session error: \(error.localizedDescription)")
    }
    
}
