//
//  TrackConfigurationInterfaceController.swift
//  TapALap
//
//  Created by Jeff Kelley on 10/30/15.
//  Copyright © 2015 The Pragmatic Bookshelf. All rights reserved.
//

import WatchKit
import Foundation


protocol TrackSelectionReceiver: class {
    func receiveTrack(track: Track)
}

class TrackConfigurationInterfaceController: WKInterfaceController {
    
    weak var trackReceiver: TrackSelectionReceiver?
    
    @IBOutlet var nameButton: WKInterfaceButton!
    @IBOutlet var lapsPicker: WKInterfacePicker!
    @IBOutlet var distancePicker: WKInterfacePicker!
    
    var selectedName: String?
    
    var selectedLaps: Int?
    var selectedDistance: Double?
    
    @IBAction func nameButtonPressed() {
        let suggestedTrackNames = ["Gym", "School", "Park", "Trail", "Neighborhood"]
        
        presentTextInputControllerWithSuggestions(suggestedTrackNames,
            allowedInputMode: .Plain) { results in
                guard let name = results?.first as? String else { return }
                
                self.nameButton.setTitle(name)
                
                self.selectedName = name
        }
    }
    
    override func awakeWithContext(context: AnyObject?) {
        super.awakeWithContext(context)
        
        if let receiver = context as? TrackSelectionReceiver {
            self.trackReceiver = receiver
        }
        
        // Add 1-10 laps for laps picker
        let lapItems: [WKPickerItem] = (1 ... 10).map { i in
            let pickerItem = WKPickerItem()
            pickerItem.title = "\(i)"
            pickerItem.caption = (i == 1) ? "Lap" : "Laps"
            return pickerItem
        }
        
        lapsPicker.setItems(lapItems)
        
        // Add 0.5 - 5 miles for total distance picker
        var distanceItems: [WKPickerItem] = []
        
        let distanceFormatter = NSLengthFormatter()
        distanceFormatter.numberFormatter.minimumFractionDigits = 1
        distanceFormatter.numberFormatter.maximumFractionDigits = 1
        
        for i in 0.5.stride(to: 5.5, by:0.5) {
            let pickerItem = WKPickerItem()
            pickerItem.title = distanceFormatter.stringFromValue(i,
                unit: .Mile)
            
            distanceItems.append(pickerItem)
        }
        
        distancePicker.setItems(distanceItems)
        
        // Set values based on initial picker values.
        lapsPickerDidChange(selectedIndex: 0)
        distancePickerDidChange(selectedIndex: 0)
    }
    
    @IBAction func lapsPickerDidChange(selectedIndex i: Int) {
        selectedLaps = i + 1
    }
    
    @IBAction func distancePickerDidChange(selectedIndex i: Int) {
        // Convert from miles to meters
        selectedDistance = Double(i + 1) / 2.0 * 1609.34
    }
    
    override func willActivate() {
        super.willActivate()
        
        lapsPicker.focus()
    }
    
    @IBAction func saveButtonPressed() {
        guard let name = selectedName else {
            presentAlertControllerWithTitle("Error",
                message: "You must select a name for your track.",
                preferredStyle: .Alert,
                actions: [
                    WKAlertAction(title: "OK",
                        style: .Default,
                        handler: {})
                ])
            
            return
        }
        
        guard let laps = selectedLaps, distance = selectedDistance else {
            fatalError("No laps/distance selected. Double-check your implementation"
                + " of awakeWithContext(_:)!")
        }
        
        let lapDistance = distance / Double(laps)
        
        let track = Track(name: name, lapDistance: lapDistance)
        
        trackReceiver?.receiveTrack(track)
    }
    
}
