//
//  ComplicationController.swift
//  NomNomNom WatchKit Extension
//
//  Created by Jeff Kelley on 10/9/15.
//  Copyright © 2015 Jeff Kelley. All rights reserved.
//

import ClockKit
import EventKit

extension EKEventStore {
    func requestNextMealEvent(forDate date: NSDate = NSDate(),
        handler: EKEvent? -> Void) {
            requestAccessToEntityType(.Event) { (success, error) in 
                if success {
                    let timePredicate = self.predicateForEventsWithStartDate(date, 
                        endDate: date.midnightAfter, calendars: nil)
                    
                    let mealPredicates = ["breakfast", "brunch", "lunch", "supper", 
                        "dinner", "snack", "coffee", "tea", "happy hour", "drinks"]
                        .map { NSPredicate(format: "title contains[cd] %@", $0) } 
                    
                    let mealMatchingPredicate = NSCompoundPredicate( 
                        orPredicateWithSubpredicates: mealPredicates)
                    
                    let events = self.eventsMatchingPredicate(timePredicate) 
                        .filter { mealMatchingPredicate.evaluateWithObject($0) } 
                        .sort { return $0.startDate.compare($1.startDate) ==
                            .OrderedAscending } 
                    
                    handler(events.first) 
                }
                else {
                    handler(nil) 
                }
            }
    }
}

class ComplicationController: NSObject, CLKComplicationDataSource {
    
    // MARK: - EventKit
    lazy var eventStore = EKEventStore()
    
    // MARK: - Timeline Configuration
    
    func getSupportedTimeTravelDirectionsForComplication(
        complication: CLKComplication,
        withHandler handler: (CLKComplicationTimeTravelDirections) -> Void) {
            handler([.Forward, .Backward])
    }
    
    func getTimelineStartDateForComplication(complication: CLKComplication,
        withHandler handler: (NSDate?) -> Void) {
            handler(nil)
    }
    
    func getTimelineEndDateForComplication(complication: CLKComplication,
        withHandler handler: (NSDate?) -> Void) {
            handler(nil)
    }
    
    func getPrivacyBehaviorForComplication(complication: CLKComplication,
        withHandler handler: (CLKComplicationPrivacyBehavior) -> Void) {
            handler(.HideOnLockScreen)
    }
    
    // MARK: - Timeline Population
    
    func getCurrentTimelineEntryForComplication(complication: CLKComplication,
        withHandler handler: ((CLKComplicationTimelineEntry?) -> Void)) {
            
            switch complication.family { 
            case .ModularLarge:
                eventStore.requestNextMealEvent { event in 
                    guard let event = event else { 
                        handler(self.modularLargeTimelineEntryForNoEvents())
                        return
                    }
                    
                    handler(self.modularLargeTimelineEntry(forEvent: event))
                }
                
            case .UtilitarianLarge:
                eventStore.requestNextMealEvent { event in
                    guard let event = event else {
                        handler(self.utilitarianLargeTimelineEntryForNoEvents())
                        return
                    }
                    
                    handler(self.utilitarianLargeTimelineEntry(forEvent: event))
                }
                
            default:
                handler(nil)
            }
    }
    
    func modularLargeTimelineEntryForNoEvents() -> CLKComplicationTimelineEntry {
        let template = CLKComplicationTemplateModularLargeStandardBody() 
        
        template.headerTextProvider = CLKSimpleTextProvider(text:
            "Food Calendar")
        
        template.headerTextProvider.tintColor = .yellowColor()
        
        template.body1TextProvider = CLKSimpleTextProvider(text:
            "No more food today")
        
        return CLKComplicationTimelineEntry(date: NSDate(), 
            complicationTemplate: template)
    }
    
    func utilitarianLargeTimelineEntryForNoEvents()
        -> CLKComplicationTimelineEntry {
            let template = CLKComplicationTemplateUtilitarianLargeFlat()
            
            template.textProvider = CLKSimpleTextProvider(text:
                "No more food today", shortText: "No more food")
            
            return CLKComplicationTimelineEntry(date: NSDate(),
                complicationTemplate: template)
    }
    
    func modularLargeTimelineEntry(forEvent event: EKEvent) ->
        CLKComplicationTimelineEntry {
            let template = CLKComplicationTemplateModularLargeStandardBody()
            
            template.headerTextProvider = CLKTimeIntervalTextProvider( 
                startDate: event.startDate, endDate: event.endDate)
            
            template.headerTextProvider.tintColor = .yellowColor()
            
            template.body1TextProvider = CLKSimpleTextProvider(text: event.title)
            
            if let location = event.location {
                template.body2TextProvider = CLKSimpleTextProvider(text: location)
            }
            
            return CLKComplicationTimelineEntry(date: event.startDate, 
                complicationTemplate: template)
    }
    
    func utilitarianLargeTimelineEntry(forEvent event: EKEvent) ->
        CLKComplicationTimelineEntry {
            let template = CLKComplicationTemplateUtilitarianLargeFlat()
            
            let timeTextProvider = CLKTimeIntervalTextProvider(
                startDate: event.startDate, endDate: event.endDate)
            let titleTextProvider = CLKSimpleTextProvider(text: event.title)
            
            template.textProvider = CLKTextProvider
                .nnn_textProviderByJoiningProvider(timeTextProvider,
                    andProvider: titleTextProvider, withString: " ")
            
            return CLKComplicationTimelineEntry(date: event.startDate,
                complicationTemplate: template)
    }
    
    func getTimelineEntriesForComplication(complication: CLKComplication,
        beforeDate date: NSDate,
        limit: Int,
        withHandler handler: (([CLKComplicationTimelineEntry]?) -> Void)) {
            // Call the handler with the timeline entries prior to the given date
            handler(nil)
    }
    
    func getTimelineEntriesForComplication(complication: CLKComplication,
        afterDate date: NSDate,
        limit: Int,
        withHandler handler: (([CLKComplicationTimelineEntry]?) -> Void)) {
            // Call the handler with the timeline entries after to the given date
            handler(nil)
    }
    
    // MARK: - Update Scheduling
    
    func getNextRequestedUpdateDateWithHandler(handler: (NSDate?) -> Void) {
        // Call the handler with the date when you would next like to be given the
        // opportunity to update your complication content
        handler(nil);
    }
    
    // MARK: - Placeholder Templates
    
    func getPlaceholderTemplateForComplication(complication: CLKComplication,
        withHandler handler: (CLKComplicationTemplate?) -> Void) {
            switch complication.family {
            case .ModularLarge:
                handler(modularLargeTimelineEntryForNoEvents().complicationTemplate)
            case .UtilitarianLarge:
                handler(
                    utilitarianLargeTimelineEntryForNoEvents().complicationTemplate)
            default:
                handler(nil)
            }
    }
    
}
